@@ -15,14 +15,16 @@
 #   nosetests .
 
 import glob
-# https://docs.python.org/3.7/library/io.html
+# https://docs.python.org/3.5/library/io.html
 import io
 import itertools
 import os
 import sys
 
-# http://www.python.org/doc/2.4.4/lib/module-unittest.html
+# https://docs.python.org/3.5/library/unittest.html
 import unittest
+# https://docs.python.org/3.8/library/warnings.html
+import warnings
 import zlib
 
 from array import array
@@ -66,30 +68,6 @@ def topngbytes(name, rows, x, y, **k):
     return f.getvalue()
 
 
-def mycallersname():
-    """Returns the name of the caller of the caller of this function
-    (hence the name of the caller of the function in which
-    "mycallersname()" textually appears).  Returns None if this cannot
-    be determined.
-    """
-
-    # http://docs.python.org/library/inspect.html#the-interpreter-stack
-    import inspect
-
-    frame = inspect.currentframe()
-    if not frame:
-        return None
-    (
-        frame_,
-        filename_,
-        lineno_,
-        funname,
-        linelist_,
-        listi_,
-    ) = inspect.getouterframes(frame)[2]
-    return funname
-
-
 class Test(unittest.TestCase):
     # This member is used by the superclass.  If we don't define a new
     # class here then when we use self.assertRaises() and the PyPNG code
@@ -260,23 +238,25 @@ class Test(unittest.TestCase):
         """Adam7 interlace reading.
         For images in the PngSuite that have both
         an interlaced and straightlaced pair,
-        test that both images of the pair give the same array of pixels."""
+        test that both images of the pair give the same array of pixels.
+        """
         for candidate in pngsuite.png:
             if not candidate.startswith("basn"):
                 continue
             candi = candidate.replace("n", "i")
             if candi not in pngsuite.png:
                 continue
-            straight = png.Reader(bytes=pngsuite.png[candidate])
-            adam7 = png.Reader(bytes=pngsuite.png[candi])
-            # Just compare the pixels.  Ignore x,y (because they're
-            # likely to be correct?); metadata is ignored because the
-            # "interlace" member differs.  Lame.
-            straight = straight.read()[2]
-            adam7 = adam7.read()[2]
-            self.assertEqual(
-                [list(row) for row in straight], [list(row) for row in adam7]
-            )
+            with self.subTest(target=candidate):
+                straight = png.Reader(bytes=pngsuite.png[candidate])
+                adam7 = png.Reader(bytes=pngsuite.png[candi])
+                # Just compare the pixels.  Ignore x,y (because they're
+                # likely to be correct?); metadata is ignored because the
+                # "interlace" member differs.  Lame.
+                straight = straight.read()[2]
+                adam7 = adam7.read()[2]
+                self.assertEqual(
+                    [list(row) for row in straight], [list(row) for row in adam7]
+                )
 
     def test_interlace_write(self):
         """Adam7 interlace writing.
@@ -290,37 +270,38 @@ class Test(unittest.TestCase):
             # Only certain colour types supported for this test.
             if name[3:5] not in ["n0", "n2", "n4", "n6"]:
                 continue
-            it = png.Reader(bytes=bytes)
-            x, y, pixels, meta = it.read()
-            pngi = topngbytes(
-                "adam7wn" + name + ".png",
-                pixels,
-                x=x,
-                y=y,
-                bitdepth=it.bitdepth,
-                greyscale=it.greyscale,
-                alpha=it.alpha,
-                transparent=it.transparent,
-                interlace=False,
-            )
-            x, y, ps, meta = png.Reader(bytes=pngi).read()
-            it = png.Reader(bytes=bytes)
-            x, y, pixels, meta = it.read()
-            pngs = topngbytes(
-                "adam7wi" + name + ".png",
-                pixels,
-                x=x,
-                y=y,
-                bitdepth=it.bitdepth,
-                greyscale=it.greyscale,
-                alpha=it.alpha,
-                transparent=it.transparent,
-                interlace=True,
-            )
-            x, y, pi, meta = png.Reader(bytes=pngs).read()
-            self.assertEqual(
-                [list(row) for row in ps], [list(row) for row in pi]
-            )
+            with self.subTest(target=name):
+                it = png.Reader(bytes=bytes)
+                x, y, pixels, meta = it.read()
+                pngi = topngbytes(
+                    "adam7wn" + name + ".png",
+                    pixels,
+                    x=x,
+                    y=y,
+                    bitdepth=it.bitdepth,
+                    greyscale=it.greyscale,
+                    alpha=it.alpha,
+                    transparent=it.transparent,
+                    interlace=False,
+                )
+                x, y, ps, meta = png.Reader(bytes=pngi).read()
+                it = png.Reader(bytes=bytes)
+                x, y, pixels, meta = it.read()
+                pngs = topngbytes(
+                    "adam7wi" + name + ".png",
+                    pixels,
+                    x=x,
+                    y=y,
+                    bitdepth=it.bitdepth,
+                    greyscale=it.greyscale,
+                    alpha=it.alpha,
+                    transparent=it.transparent,
+                    interlace=True,
+                )
+                x, y, pi, meta = png.Reader(bytes=pngs).read()
+                self.assertEqual(
+                    [list(row) for row in ps], [list(row) for row in pi]
+                )
 
     def test_interlace_write_array_bytes(self):
         """
@@ -356,6 +337,24 @@ class Test(unittest.TestCase):
         # Same palette
         self.assertEqual(again_info["palette"], info["palette"])
 
+    def test_read_palette_RGB_write(self):
+        """Test that a paletted PNG can be read asRGB() and
+        that info dict can be used correctly in a Writer.
+        See https://gitlab.com/drj11/pypng/-/issues/121
+        """
+
+        r = png.Reader(bytes=pngsuite.basn3p04)
+        x, y, pixels, info = r.asRGB()
+        w = png.Writer(**info)
+        o = BytesIO()
+        w.write(o, pixels)
+        o.flush()
+        o.seek(0)
+        r = png.Reader(file=o)
+        _, _, _, again_info = r.read()
+        # Same palette
+        self.assertEqual(again_info["palette"], info["palette"])
+
     def test_deepen_palette(self):
         """Test that palette bitdepth can be increased,
         without change of pixel values."""
@@ -737,11 +736,12 @@ class Test(unittest.TestCase):
         bad_png = bytearray(pngsuite.basi0g01)
         bad_png[30] = 1
         r = png.Reader(bytes=bad_png)
-        try:
+
+        with warnings.catch_warnings(record=True) as w:
             r.read()
-        except Exception as e:
-            self.assertIsInstance(e, png.FormatError)
-            self.assertIn("hecksum error", str(e))
+            assert len(w) == 1
+            assert issubclass(w[0].category, Warning)
+            self.assertIn("hecksum error", str(w[0].message))
 
     def test_type_bad(self):
         """
@@ -1228,7 +1228,7 @@ class Test(unittest.TestCase):
 
     def test_writer_height_bad(self):
         """Invoking Writer with bad height should raise error."""
-        self.assertRaises(png.ProtocolError, png.Writer, -4, 4)
+        self.assertRaises(png.ProtocolError, png.Writer, 4, -4)
 
     def test_writer_width_big(self):
         """Invoking Writer with big width should raise error."""
@@ -1241,7 +1241,10 @@ class Test(unittest.TestCase):
     # scripts in test directory
 
     def test_test_dir(self):
-        """Runs each executable in the test directory."""
+        """
+        Runs each executable in the test directory
+        as a subTest.
+        """
 
         test_path = os.path.join(os.path.dirname(__file__), "test")
 
@@ -1261,10 +1264,9 @@ class Test(unittest.TestCase):
             if not os.access(path, os.X_OK):
                 # Skip non-executables (probably fixtures?)
                 continue
-            status = os.system(path)
-            runs.append((path, status))
-        failed_runs = [run for run in runs if run[1]]
-        self.assertTrue(len(failed_runs) == 0, msg="%r failed" % failed_runs)
+            with self.subTest(target=path):
+                status = os.system(path)
+                self.assertEqual(status, 0)
 
 
 def read_modify_chunks(modify_chunk):
